  .model small,c
  .code

; norm_esdi
; ---------
;
; internal -- normalises an address in es:di.
;
; trashes ax, cx, and dx.

norm_esdi proc

  mov dx,es

  mov cl,4
  mov ax,di
  shr ax,cl
  add dx,ax
  and di,0fh
  mov es,dx
  ret

norm_esdi endp

; norm_dssi
; ---------
;
; internal -- normalises an address in ds:si.
;
; trashes ax, cx, and dx.

norm_dssi proc

  mov dx,ds

  mov cl,4
  mov ax,si
  shr ax,cl
  add dx,ax
  and si,0fh
  mov ds,dx
  ret

norm_dssi endp

; void farmemcpy(void far *dst, void far *src, word num)
; ------------------------------------------------------
;
; copies up to almost a segment of memory from one place to another.

farmemcpy proc uses si di es ds, dst:far ptr byte, src: far ptr byte, num: word
public farmemcpy

  lds si, src
  les di, dst
  call norm_dssi
  call norm_esdi

  mov cx, word ptr [num]

  cld
  rep movsb                 ; copy the bytes

  ret

farmemcpy endp

; unsigned farstrlen(void far *src)
; ---------------------------------
;
; returns the length of a far asciiz string. limited to strings at most
; about 64k long (within a paragraph, anyway).

farstrlen proc uses es di, src: far ptr byte returns len: word
public farstrlen

  les di, src

  call norm_esdi
  mov bx, di ; save that one for later

  mov cx, 0ffffh
  mov al, 0
  cld
  repne scasb
  jne short fsl_nonsensible
  mov ax, di
  sub ax, bx ; that should give us the string length
  dec ax     ; on the other hand, maybe I should subtract one.
  ret

fsl_nonsensible:

  xor ax, ax
  ret

farstrlen endp

; int farstrcmp(void far *dst, void far *src)
; -------------------------------------------
;
; far string comparisons. both strings should be less than 4095 paragraphs
; (65520) bytes in length, and null-terminated.
;
; returns:  0 == match
;          -1 == src<dest
;           1 == src>dest
;
; trashes: ax, bx, cx, dx

farstrcmp proc uses si di es ds, src:far ptr byte, dst: far ptr byte returns res: word
public farstrcmp

  les di, src           ; get source address
  call norm_esdi

  mov bx, di
  mov cx, 0ffffh
  mov al, 0
  cld
  repne scasb
  jne short fsc_match
  sub di, bx
  push di               ; push the string length to the stack

  lds si, src           ; get source address...
  call norm_dssi        ; ... and normalise it

  les di, dst           ; "destination" address...
  call norm_esdi        ; ... and normalise it

  pop cx                ; use saved length as the maximum

  repe cmpsb            ; do the comparison

  je fsc_match
  jc fsc_smaller

  mov ax, 1
  ret

fsc_smaller:

  mov ax, 0ffffh
  ret

fsc_match:

  mov ax, 0h
  ret

farstrcmp endp

; int farstrncmp(void far *dst, void far *src, word num)
; ------------------------------------------------------
;
; far string comparisons, up to a specified length. both strings should be
; less than 4095 paragraphs (65520) bytes in length.
;
; returns:  0 == match
;          -1 == src<dest
;           1 == src>dest
;
; trashes: ax, bx, cx, dx
;
; WARNING: no attention whatsoever is paid to the null-termination of
;          the strings being passed.

farstrncmp proc uses si di es ds, src:far ptr byte, dst: far ptr byte, num: word returns res: word
public farstrncmp

  lds si, src
  les di, dst

  call norm_esdi
  call norm_dssi

  mov cx, word ptr [num]

  cld
  repe cmpsb
  je fsc_match          ; WARNING:
  jc fsc_smaller        ;   depends on farstrcmp()

  mov ax, 1
  ret

farstrncmp endp

; void far *get_mcb(void)
; -----------------------
;
; finds the top of the mcb chain

get_mcb proc
public get_mcb

  mov ah, 52h
  int 21h
  mov dx, es:[bx-2]
  xor ax, ax

  ret

get_mcb endp

; char far *nxtevar(char far *vptr)
; ---------------------------------
;
; returns a far pointer to the byte immediately following the first null
; character in the argument string, or NULL if the string is empty.
;
; as usual, the limitation on string length is about a paragraph short of
; 64k.

nxtevar proc uses di, vptr:far ptr byte returns nextenv:far ptr byte
public nxtevar

  les di, vptr
  call norm_esdi

  mov cx, 0ffffh
  xor ax, ax      ; search for 0 and...
  mov dx, ax      ; ...initialize return DX:AX to 0:0
  repne scasb     ; search ES:DI for char 0 in AL
  cmp cx, 0fffeh  ;
  je nev

  call norm_esdi
  mov dx, es
  mov ax, di

nev:

  ret

nxtevar endp

; char far *par_env(int far *envsize)

par_env proc uses es di, envsize:far ptr byte returns env:far ptr byte
public par_env

  mov ah, 62h                   ; get psp address of this process
  int 21h
  mov es, bx
  mov di, es:[16h]              ; di = psp of parent

  ; idiot-checking: does this mcb actually control a psp?
  ;
  ; or: is the owner of this mcb the psp it controls?

  dec di
  mov es, di                    ; es = mcb of psp of parent
  inc di                        
  mov ax, word ptr es:[1]       ; ax = owner of mcb
  cmp ax, di                    ; is owner of mcb == psp?
  je pe_is_parmcb

  ; return NULL

pe_null:

  xor ax, ax
  mov dx, ax

  ret

pe_is_parmcb:

  mov cx, di                    ; store the psp of the parent
  mov es, di                    ; es = psp of parent
  mov di, word ptr es:[2ch]     ; ax = env of parent
  or di, di
  je pe_null                    ; if it's null, zonk out

  dec di
  mov es, di                    ; es = mcb of env of parent

  ; more idiot proofing -- make sure that the segment address of
  ; the environment makes sense, by checking the mcb it should
  ; have.

  cmp byte ptr es:[0], 'M'
  je pe_is_mcb
  cmp byte ptr es:[0], 'Z'
  jne pe_null

pe_is_mcb:

  ; still more idiot proofing -- who =owns= the mcb? if it's
  ; not the parent, puke.

  mov ax, word ptr es:[1]       ; owner of mcb of env of parent
  cmp ax, cx                    ; (we saved this earlier -- cx = psp of parent)
  jne pe_null                   ; hope it's within range!

  ; now we're this far -- they might want the environment
  ; size passed back to them somehow.

  mov cx, word ptr es:[3]       ; cx = size of env of parent
  mov dx, es
  inc dx                        ; dx = env of parent

  les di, envsize
  mov ax, es
  je pe_happy

  mov es:[di], cx               ; give them the env size in paragraphs

  ; return in dx:ax, and we already have dx set up properly, so...

pe_happy:

  xor ax, ax
  ret

par_env endp



  end

