comment | USE WITH QB/PDS.  Assemble with MASM 5.1 or better.

Written by Brian McLaughlin. Released into public domain 1/5/94.
This source code contains two integer FUNCTIONs: SetBit% and GetBit%.

SetBit accepts a target integer and a second paramenter naming the
target bit (0-15) to set or clear, and a third parameter showing
whether to set or to clear the target bit. To clear the target bit, the
third value should be zero. To set the target bit use any non-zero
value. Because SetBit is written as a FUNCTION, the original target
integer WILL NOT be altered, unless you reassign it, using the
value returned by SetBit, like so:

   Target% = SetBit%(Target%, 0, 0)    'sets bit 0 to 0 in Target%

NOTE: Bit position 0 is the furthest "righthand" bit in the Target%,
when it is written in binary notation.

GetBit accepts a target integer, and a second parameter that shows
which bit (0-15) to read. GetBit returns a zero if the bit is zero,
or -1 if the bit is 1. This lets you use IF NOT GetBit%...THEN.

IMPORTANT NOTE: Both these FUNCTIONs give erratic results when passed a
bit position other than 0 through 15.

DECLARE them:     DECLARE FUNCTION SetBit%(Target%, BitPos%, SetOrClr%)
                  DECLARE FUNCTION GetBit%(Target%, BitPos%)
end comment |

.MODEL MEDIUM, BASIC
.CODE

SetBit PROC FAR USES DI, Intgr:WORD, BitPos:WORD, SetIt:WORD
    Mov BX, Intgr    ;BX = address of the target integer
    Mov AX, [BX]     ;AX = value of target integer
    Mov BX, SetIt    ;BX = address of SetIt
    Mov DX, [BX]     ;DX = value of SetIt
    Mov BX, BitPos   ;BX = address of BitPos
    Mov CX, [BX]     ;CX = value of BitPos
    Mov DI, 1        ;set mask (DI) to 00000000 00000001b
    Shl DI, CL       ;DI holds the mask, shift it left CL times
    Cmp DX, 0        ;should the bit be cleared?
    Je  ClrBit       ;if so, then skip ahead
    Or  AX, DI       ;set bit to 1 by ORing with the mask
    Jmp SHORT Exit   ;and return
ClrBit:
    Not DI           ;to clear bit, reverse the mask
    And AX, DI       ;then AND the mask and the target integer
Exit:
    Ret              ;all done!
SetBit ENDP

GetBit PROC FAR USES DI, Intgr:WORD, BitPos:WORD
    Xor AX, AX       ;assume we're returning a zero in AX
    Mov BX, Intgr    ;BX = address of the target integer
    Mov DX, [BX]     ;DX = value of target integer
    Mov BX, BitPos   ;BX = address of BitPos
    Mov CX, [BX]     ;CX = value of BitPos
    Mov DI, 1        ;set mask (DI) to 00000000 00000001b
    Shl DI, CL       ;DI holds the mask, shift it CL times
    And DI, DX       ;AND the mask and the target integer
    Jz  AllDone      ;if zero, the bit WAS clear: return 0 in AX
    Dec AX           ;if not, we need to return a -1 in AX
AllDone:
    Ret
GetBit ENDP
    END
