                Name    Clock
                Page    ,132
comment~
===============================================================================

        Routine should be declared in QB/QBX like so..
        DECLARE SUB Clock (row%, col%, attr%, onoff%)
                            |     |      |      |
        Screen row----------'     |      |      |
        Screen column-------------'      |      |
        Color atrribute------------------'      |
        On/Off boolean, [0 or 1]----------------' 

==============================================================================~

                .model  medium,basic
                public  clock
Code            Segment Byte Public 'Code'
                Assume  CS:Code, DS:Code

;------------------------------------------------------------------------------
;  Data area
;------------------------------------------------------------------------------

Adapter         DB 0                            ;1 = MDA, 2 = CGA, 3 = EGA
Attribute       DB ?                            ;requested color
Clock_Text      DB ?,?,58,?,?,58,?,?,32,?,109   ;HH:MM:SS am (or pm)
Counter         DB ?                            ;0 when time for update
Old_INT1C       DW ?,?                          ;storage for old INT1C vector
Status          DB 0                            ;0 = off, 1 = on
Video_Buffer    DW ?                            ;0B00h or 0B800h
Video_Offset    DW ?                            ;location for screen display

Time            STRUC
                Hrs     DB ?                    ;storage for hours,
                Min     DB ?                    ;  minutes and
                Sec     DB ?                    ;  seconds
Time            Ends

Current_Time    Time    <>                      ;DB 3 DUP(?)

;------------------------------------------------------------------------------
;  Execution comes here when program is CALLed from within BASIC.
;------------------------------------------------------------------------------

Clock           Proc    Far

Begin:          PUSH    BP                      ;save BP for BASIC
                MOV     BP,SP                   ;get stack address
                MOV     SI,[BP+12]              ;get row% into DH
                MOV     DH,[SI]
                DEC     DH                      ;adjust 1-25 to 0-24
                MOV     SI,[BP+10]              ;put column% into DL
                MOV     DL,[SI]
                DEC     DL                      ;adjust 1-70 to 0-69
                MOV     SI,[BP+08]              ;put color% into BL
                MOV     BL,[SI]
                MOV     SI,[BP+06]              ;put mode% into BH
                MOV     BH,[SI]

                PUSH    DS                      ;save BASIC's data segment
                PUSH    CS                      ;set DS to the code segment
                POP     DS                      ;so we can find our data

                CMP     BH,0                    ;termination requested ?
                JE      Terminate               ;  yes, restore INT 1Ch address
                CMP     BH,1                    ;installation requested ?
                JNE     Exit                    ;  no, exit now

; Initialize our internal variables.  Then get and display the current time.

                CALL    Initialize

                MOV     AH,2Ch                  ;get the current time
                INT     21h                     ;call DOS to get it
                MOV     Current_Time.Hrs,CH     ;save hours
                MOV     Current_Time.Min,CL     ;save minutes
                MOV     Current_Time.Sec,DH     ;save seconds

                MOV     Counter,1               ;always display time at start-up
                CALL    Do_Clock                ;display the current time

; Check to see if we're already installed.  If not, save the ROM interrupt 1Ch
; vector and replace it with one pointing to our code.

                CMP     Status,1                ;are we installed ?
                JE      Exit                    ;  yes, exit now
                MOV     AX,351Ch                ;get current interrupt 1Ch vector
                INT     21h                     ;call DOS to do it
                MOV     Old_INT1C,BX            ;save vector offset
                MOV     Old_INT1C[2],ES         ;save vector segment

                MOV     AX,251Ch                ;set new interrupt 1Ch vector
                LEA     DX,INT1C                ;point it to our code
                INT     21h                     ;call DOS to do it
                MOV     Status,1                ;indicate we're on now
                JMP     SHORT Exit              ;return to BASIC

; Execution jumps here on CALL from BASIC with mode%=0.

Terminate:      CMP     Status,1                ;are we installed ?
                JNE     Exit                    ;  no, exit now
                MOV     Status,0                ;indicate we're turning off
                                                ;put ROM INT 1C vector into DS:DX
                LDS     DX,DWORD PTR Old_INT1C
                MOV     AX,251Ch                ;reset interrupt 1Ch vector
                INT     21h                     ;call DOS to do it

;------------------------------------------------------------------------------
; Program exit is achieved here when CALLed from within BASIC.
;------------------------------------------------------------------------------

Exit:           POP     DS                      ;restore BASIC's data segment
                POP     BP                      ;restore BP for BASIC
                RET     8                       ;return to BASIC discarding parms
Clock           Endp

;------------------------------------------------------------------------------
; Once installed, execution comes here with each interrupt 1Ch.
;------------------------------------------------------------------------------

INT1C:          CALL    Do_Clock                ;update clock if it's time
                IRET                            ;return to interrupted program

;------------------------------------------------------------------------------
; DO_CLOCK increments Current_Time by 1 second once every 18 times it is
; called.  The new time is sent to the display.
;------------------------------------------------------------------------------

Do_Clock        Proc    Near
                PUSH    AX                      ;save all registers
                PUSH    CX
                PUSH    DX
                PUSH    SI
                PUSH    DI
                PUSH    DS
                PUSH    ES

                PUSH    CS                      ;set DS to the code segment
                POP     DS                      ;so we can find our data
                STI                             ;allow further interrupts
                DEC     Counter                 ;show we're here
                JNZ     Done                    ;if not to zero, exit

                MOV     CL,60                   ;constant divisor
                MOV     DX,0C0Ah                ;constant divisors
                LEA     DI,Clock_Text[6]        ;do seconds first

                MOV     AL,Current_Time.Sec     ;seconds in AL
                INC     AL                      ;add 1 second
                CBW                             ;byte to word in AX
                DIV     CL                      ;divide seconds by 60
                MOV     Current_Time.Sec,AH     ;save seconds
                CALL    Bin_to_ASCII            ;write seconds to Clock_Text

                MOV     AL,Current_Time.Min     ;minutes in AL
                ADD     AL,CH                   ;add seconds quotient
                CBW                             ;byte to word in AX
                DIV     CL                      ;divide minutes by 60
                MOV     Current_Time.Min,AH     ;save minutes
                CALL    Bin_to_ASCII            ;write minutes to Clock_Text

                MOV     AL,Current_Time.Hrs     ;hours in AL
                ADD     AL,CH                   ;add minutes quotient
                MOV     Current_Time.Hrs,AL     ;save hours
                CBW                             ;byte to word in AX
                DIV     DH                      ;correct for hours > 12
                CMP     AH,0                    ;noon or midnight ?
                JNE     Cont                    ;  no, continue
                MOV     AH,12                   ;  yes, change 0 to 12
Cont:           CALL    Bin_to_ASCII            ;write hours to Clock_Text

                MOV     Clock_Text[9],'p'       ;assume pm for now
                CMP     CH,1                    ;afternoon or evening (pm) ?
                JE      Write_Time              ;  yes, display the time
                MOV     Clock_Text[9],'a'       ;  no, write 'am'

Write_Time:     MOV     ES,Video_Buffer         ;set ES to screen segment
                LEA     SI,Clock_Text           ;point SI to text storage
                MOV     AH,Attribute            ;color to use in AH
                MOV     DI,Video_Offset         ;get display location
                MOV     DX,3DAh                 ;CGA mode status register in DX
                MOV     CX,11                   ;11 characters to print
                CLD                             ;clear DF for string moves

Output1:        CMP     Adapter,2               ;are we doing CGA ?
                JNE     Output2                 ;  if not, skip the retrace stuff
No_Retrace:     IN      AL,DX                   ;get the video status byte
                TEST    AL,1                    ;horizontal retrace active ?
                JNE     No_Retrace              ;  yes, wait until it's over
Retrace:        IN      AL,DX                   ;get the video status byte again
                TEST    AL,1                    ;horizontal retrace active ?
                JE      Retrace                 ;  no, wait until the next one
Output2:        LODSB                           ;character in AL
                STOSW                           ;display character/attribute pair
                LOOP    Output1                 ;continue until done CX times

                MOV     Counter,18              ;start count over again

Done:           POP     ES                      ;restore all registers
                POP     DS
                POP     DI
                POP     SI
                POP     DX
                POP     CX
                POP     AX
                RET
Do_Clock        Endp

;------------------------------------------------------------------------------
; BIN_TO_ASCII converts a single hex byte to its 2 character ASCII equivalent
; and stores those characters in Clock_Text.
;------------------------------------------------------------------------------

Bin_to_ASCII    Proc    Near
                MOV     CH,AL                   ;save prior quotient in CH
                MOV     AL,AH                   ;put number into AL
                CBW                             ;byte to word in AX
                DIV     DL                      ;divide AL by 10
                ADD     AL,48                   ;convert quotient to ASCII
                MOV     [DI],AL                 ;store digit in Clock_Text
                INC     DI                      ;point to next digit
                ADD     AH,48                   ;convert remainder to ASCII
                MOV     [DI],AH                 ;store digit in Clock_Text
                SUB     DI,4                    ;point DI to 3rd prior digit
                RET
Bin_to_ASCII    Endp

;------------------------------------------------------------------------------
; INITIALIZE sets the values of various parameters required during execution.
;------------------------------------------------------------------------------

Initialize      Proc    Near

; derive the offset address in video memory for location row%, column%.

                MOV     AL,160                  ;160 bytes per row
                MUL     DH                      ;AX = offset value of row%
                SHL     DL,1                    ;column% * 2
                XOR     DH,DH                   ;byte to word in DX
                ADD     AX,DX                   ;add the two
                MOV     Video_Offset,AX         ;store offset internally

                MOV     Attribute,BL            ;store color% internally

; identify the display type and set monitor dependent values accordingly.

                CMP     Adapter,0               ;have we done this before ?
                JNE     Init_Exit               ;  yes, don't repeat
                MOV     AX,40h                  ;set ES to the BIOS data segment
                MOV     ES,AX
                MOV     BX,0B000h               ;set monochrome segment for now
                MOV     DL,1                    ;set monochrome adapter for now
                MOV     AL,ES:[10h]             ;get equipment list low byte
                AND     AL,110000b              ;clear bits 0 - 3
                CMP     AL,30h                  ;monochrome adapter ?
                JE      Save_Values             ;  yes, save values
                ADD     BX,800h                 ;  no, set BX to color segment
                INC     DL                      ;  and increase DL to 2
                MOV     AL,ES:[87h]
                CMP     AL,0                    ;is an EGA installed ?
                JE      Save_Values             ;  no, leave DL set to 2
                INC     DL                      ;  yes, increase DL to 3
Save_Values:    MOV     Video_Buffer,BX         ;save the screen buffer address
                MOV     Adapter,DL              ;save our adapter code
Init_Exit:      RET
Initialize      Endp

Code            Ends
                End