'FDATETIM.BAS       robert.jacobson@dazed.chasnet.org
'
'Purpose...To return the date and time of a specified file.
'
'Syntax...
'   GetFileDateAndTime FileSpec$, FileTime$, FileDay$, gfdtERROR, gfdtERR$
'
'   Where:
'          FileSpec$ is the legal pathname of the file for which to retrieve
'               the date and time. (Same as DOS)
'          FileTime$, FileDay$, gfdtERROR and gfdtERR$ are values
'               returned by the subprogram.
'          FileTime$ and FileDay$ are string formatted similar to the
'               DOS directory listing format.
'
'Invoke QB45 at the DOS prompt using...
'
'       QB /L
'
'This will automatically load the QB.QLB required to use the INTERRUPT
'   function.
'
'This program will NOT run in QBasic. Sorry, you'll have to write an
'   assembly language routine (to replace the INTERRUPT functions) and use
'   the CALL ABSOLUTE command.

'This program does NOT create a file if the file is not present. It returns
'   a "no file found error" message instead.

DEFINT A-Z

TYPE RegType
	ax AS INTEGER
	bx AS INTEGER
	cx AS INTEGER
	dx AS INTEGER
	bp AS INTEGER
	si AS INTEGER
	di AS INTEGER
 flags AS INTEGER
	ds AS INTEGER
	es AS INTEGER
END TYPE

DIM SHARED inregs AS RegType, outregs AS RegType

DECLARE SUB INTERRUPT (IntNumber%, inregs AS RegType, outregs AS RegType)
DECLARE SUB GetFileDateAndTime (UserFileSpec$, FileTime$, FileDay$, gfdtERROR, gfdtERR$)

FileSpec$ = "C:\AUTOEXEC.BAT"

GetFileDateAndTime FileSpec$, FileTime$, FileDay$, gfdtERROR, gfdtERR$
IF gfdtERROR <> 0 THEN PRINT gfdtERR$: END
PRINT FileDay$; "  "; FileTime$

SUB GetFileDateAndTime (UserFileSpec$, FileTime$, FileDay$, gfdtERROR, gfdtERR$)
'Convert UserFileSpec$ to ASCII Zero terminated string...
		DOSfileDESCRIPTOR$ = UserFileSpec$ + CHR$(0)

'Open the file for access read only...
		inregs.ax = &H3D00
		inregs.ds = VARSEG(DOSfileDESCRIPTOR$)
		inregs.dx = SADD(DOSfileDESCRIPTOR$)
		CALL INTERRUPT(&H21, inregs, outregs)
			IF outregs.flags AND 1 THEN
				SELECT CASE outregs.ax
					CASE 1
						'invalid function
						gfdtERR$ = "File Open ERROR...Invalid function"
						gfdtERROR = 1
					CASE 2
						'file not found
						gfdtERR$ = "File Open ERROR...File not found"
						gfdtERROR = 2
					CASE 3
						'path not found
						gfdtERR$ = "File Open ERROR...Path not found"
						gfdtERROR = 3
					CASE 4
						'no file handles available
						gfdtERR$ = "File Open ERROR...No file handles available"
						gfdtERROR = 4
					CASE 5
						'access denied
						gfdtERR$ = "File Open ERROR...Access denied"
						gfdtERROR = 5
					CASE &HC
						'invalid access code
						gfdtERR$ = "File Open ERROR...Invalid access code"
						gfdtERROR = 6
				END SELECT
				EXIT SUB
			ELSE
				Handle% = outregs.ax
		END IF

'Get the file's date and time...
		inregs.ax = &H5700
		inregs.bx = Handle%
		CALL INTERRUPT(&H21, inregs, outregs)
		IF outregs.flags AND 1 THEN
			SELECT CASE outregs.ax
				CASE 1
					'invalid function
					gfdtERR$ = "Get Date and Time ERROR...Invalid function..."
					gfdtERROR = 7
				CASE 6
					'invalid handle
					gfdtERR$ = "Get Date and Time ERROR...Invalid handle..."
					gfdtERROR = 8
			END SELECT
		ELSE
			FileTyme% = outregs.cx
			FileDate% = outregs.dx
		END IF

'Close the file...
		inregs.ax = &H3E00
		inregs.bx = Handle%
		CALL INTERRUPT(&H21, inregs, outregs)
		IF outregs.flags AND 1 THEN
			IF outregs.ax = 6 THEN
				'invalid handle
				gfdtERR$ = gfdtERR$ + "Close File ERROR...Invalid Handle"
				gfdtERROR = 9
			ELSE
				'some other undocumented error occurred
				gfdtERR$ = gfdtERR$ + "Close File ERROR...Some other undocumented error occurred"
				gfdtERROR = 10
			END IF
			gfdtERR$ = gfdtERR$ + "... File still open!!!"
		END IF
   
	'DATE DECODING...
	' ------------- BITS---------------
	' |F|E|D|C|B|A|9|8|7|6|5|4|3|2|1|0|
	'  * * * * * * *                     = Year plus 1980
	'                * * * *             = Month (1 to 12)
	'                        * * * * *   = Day (1 to 31)

	Year% = FileDate% \ 512 + 1980
	Year$ = RIGHT$(STR$(Year%), 2)
	Month% = (FileDate% MOD 512) \ 32
	Month$ = LTRIM$(STR$(Month%))
	IF LEN(Month$) = 1 THEN Month$ = " " + Month$
	Day% = FileDate% MOD 32
	Day$ = LTRIM$(STR$(Day%))
	IF LEN(Day$) = 1 THEN Day$ = "0" + Day$
	FileDay$ = Month$ + "-" + Day$ + "-" + Year$
   
	'TIME DECODING...
	' ------------- BITS---------------
	' |F|E|D|C|B|A|9|8|7|6|5|4|3|2|1|0|
	'  * * * * *                         = Hour (0 - 23)
	'            * * * * * *             = Minute (0 - 59)
	'                        * * * * *   = Seconds (0 - 29) (Multiply by
	'                                           two for actual seconds)

	Hour% = ((FileTyme% AND 63488)) \ 2048
	IF Hour% > 11 THEN AMpm$ = "p" ELSE AMpm$ = "a"
	IF Hour% = 0 THEN Hour% = 12
	IF Hour% > 12 THEN Hour% = Hour% - 12
	Hour$ = LTRIM$(STR$(Hour%))
	IF LEN(Hour$) = 1 THEN Hour$ = " " + Hour$
	Minute% = (FileTyme% AND 2016) \ 32
	Minute$ = LTRIM$(STR$(Minute%))
	IF LEN(Minute$) = 1 THEN Minute$ = "0" + Minute$
	Seconds% = (FileTyme% AND 31) * 2
	Seconds$ = LTRIM$(STR$(Seconds%))
	IF LEN(Seconds$) = 1 THEN Seconds$ = "0" + Seconds$
	FileTime$ = Hour$ + ":" + Minute$ + ":" + Seconds$ + AMpm$
END SUB

