/* FDUMP.C   File dump utility   30 Jan 88   Bob Jarvis  */
/*                                                       */
/* Invocation:  FDUMP <infile> (<outfile>)               */
/*    infile  - file to be dumped.  Must be given.       */
/*    outfile - output destination.  Optional.           */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys\types.h>
#include <sys\stat.h>

#define SI  15       /* Selects compressed mode on an Epson FX-85 */
#define DC2 18       /* Cancels compressed mode                   */

char graphline[65];

main(int argc, char *argv[])
   {
   FILE *infile, *outfile;
   unsigned long current_addr = 0L, infsize;
   int retval, c, i, line_length = 16;
   struct stat statbuf;

   if(argc < 2)   /* Missing the input file name argument - bitch and die */
      {
      puts("Usage: fdump <infile> (<outfile>)\n");
      exit(1);
      }

   infile = fopen(argv[1], "rb");    /* Open the input file */
   if(infile == NULL)
      {
      printf("*** Unable to open input file '%s' ***\n", argv[1]);
      exit(2);
      }

   /* Get the size of the input file */

   retval = fstat(fileno(infile), &statbuf);
   if(retval != 0)
      infsize = 0L;
   else
      infsize = statbuf.st_size;

   if(argc >= 3)  /* Open the output file */
      {
      /* First, see if the user wants to use a 'standard' system file.   */
      /* We can check for 'prn', 'con', or 'aux' - add others if needed. */

      if(memicmp(argv[2], "prn", 3) == 0)
         {
         outfile = stdprn;
         line_length = 32;
         fputc(SI, outfile);
         }
      else if(memicmp(argv[2], "con", 3) == 0)
         outfile = stdout;
      else if(memicmp(argv[2], "aux", 3) == 0)
         outfile = stdaux;
      else
         {
         outfile = fopen(argv[2], "w");
         if(outfile == NULL)
            {
            printf("*** Unable to open output file '%s' ***\n", argv[2]);
            exit(3);
            }
         }
      }
   else     /* Default to using stdout */
      outfile = stdout;

   /* Produce the header */

   fprintf(outfile, "Dump of %s (%ld bytes)\n\n",
                    strupr(argv[1]), infsize);

   /* Now, start processing in the file. */

   graphline[line_length+1] = '\0';

   c = fgetc(infile);

   while(!feof(infile))
      {
      /* First, print the current address (up to 8 digits) */

      fprintf(outfile, "%08lX: ", current_addr);
      current_addr += (long)line_length;

      for(i = 0 ; !feof(infile) && i++ < line_length ; )
         {
         fprintf(outfile, "%02X", c);
         if(i % 4 == 0)
            fputc(' ', outfile);

         if(isprint(c))
            graphline[i-1] = c;
         else
            graphline[i-1] = '.';

         c = fgetc(infile);
         }

      if(i < line_length)
         while(i++ < line_length)
            {
            fprintf(outfile, "  ");
            if(i % 4 == 0)
               fputc(' ', outfile);
            }

      fprintf(outfile, "  |%s|", graphline);

      for(i = 0 ; i < line_length ; ++i)
         graphline[i] = '\0';

      fputc('\n', outfile);
      }

   if(outfile == stdprn)
      fputc(DC2, outfile);
   }