' This file defines the structures for all the packet files used in
' FidoNet (TM), as well as the standard message header and attribute bits.

' The packet file will look like this:

' PacketHeader -> MessageHeader -> ToName -> FromName -> Subject -> MsgText

' ToName and FromName may be up to 36 bytes and are null terminated.
' Subject may be up to 72 bytes and is null terminated.
' MsgText is unbounded and is null terminated.  
' The packet will end with two null bytes.

' To figure out what type of header the .PKT has:
'
' If the "Baud" field equals in the FSC-0039 packet structure is 2, then the 
' .PKT uses a FSC-0045 packet header
' Otherwise the packet uses a FSC-0039 or FTS-0001 packet structure.  The two
' are compatable as long as you use the "QorigZone" and "QdestZone" fields.

' The "Version" field should always equal 2.

TYPE FSC0045
        origNode        AS INTEGER      'original node
        destNode        AS INTEGER      'destination node
        origPoint       AS INTEGER      'original point
        destPoint       AS INTEGER      'destination point
        zeros           AS STRING * 8   'must be 0's
        SubVersion      AS INTEGER      'packet sub-version (2 in this case)
        Version         AS INTEGER      'packet version (2 in this case)
        origNet         AS INTEGER      'original net
        destNet         AS INTEGER      'destinaition net
        PodCode         AS STRING * 1   'product code
        RevisionLvl     AS STRING * 1   'product revision level
        password        AS STRING * 8   'password
        origZone        AS INTEGER      'original zone
        destZone        AS INTEGER      'destination zone
        origDomain      AS STRING * 8   'original domain
        destDomain      AS STRING * 8   'destination domain
        PodData         AS LONG         'product specific data
END TYPE

TYPE FSC0039
        origNode        AS INTEGER      'original node
        destNode        AS INTEGER      'destination node
        year            AS INTEGER      'year
        month           AS INTEGEr      'month
        day             AS INTEGER      'day
        hour            AS INTEGER      'hour
        minute          AS INTEGER      'minute
        second          AS INTEGER      'second
        Baud            AS INTEGER      'baud rate, not in use
        Version         AS INTEGER      'should be 2
        origNet         AS INTEGER      'original net
        destNet         AS INTEGER      'destination net
        PodCodeLow      AS STRING * 1   'product code
        MajorLvl        AS STRING * 1   'product revision level
        password        AS STRING * 8   'password
        QorigZone       AS INTEGER      'original zone
        QdestZone       AS INTEGER      'destination zone
        Filler          AS STRING * 4   'spare change
        PodCodeHigh     AS STRING * 1   'product code, high order
        MinorLvl        AS STRING * 1   'product revision level, minor
        CapWord         AS INTEGER      'bitfield for capabilities
        origZone        AS INTEGER      'original zone
        destZone        AS INTEGER      'destination zone
        origPoint       AS INTEGER      'original point
        destPoint       AS INTEGER      'destination point
        PodData         AS LONG         'Product data
'        Terminator      AS STRING * 1   'packet terminator, zeros
END TYPE

TYPE MessageHeader
        Header      AS STRING * 1  'first byte in field, no longer used
                                   'will always be 02H
        NullStr     AS STRING * 1  'not used either.  Will always be NULL
        OrigNode    AS INTEGER     'original node
        DestNode    AS INTEGER     'destinaion node
        OrigNet     AS INTEGER     'original net
        DestNet     AS INTEGER     'destination net
        Attributes  AS INTEGER     'attribute bits, link FREQ, Crash, etc.
        Cost        AS INTEGER     'cost
        DateTime    AS STRING * 20 'DateTime string
End Type

' this structure will be followed by:
' To name (max 36 bytes, null terminated)
' From name (max 36 bytes, null terminated)
' subject  (max 72 bytes, null terminted)

CONST Private           = 1     'message is private
CONST Crash             = 2     'crash send
CONST BeenRead          = 4     'message has been read
CONST Sent              = 8     'message is sent
CONST Attach            = 16    'file is attached
CONST Forward           = 32    'being forwarded
CONST UnknownDest       = 64    'destination node is unknown
CONST KillSen           = 128   'kill after mailing
CONST IsLocal           = 256   'FidoNet vs. local
CONST NothingYet        = 512   'expansion bit
CONST NothingYet2       = 1024  'ditto
CONST Freq              = 2048  'file request
CONST RecieptRqst       = 4096  'receipt requested
CONST IsReciept         = 8192  'is a return reciept
CONST AuditRqst         = 16384 'audit trail requested
CONST UpdateRqst        = 32768 'update request

TYPE Address         'FidoNet 4D style address
   Zone  AS INTEGER
   Net   AS INTEGER
   Node  AS INTEGER
   Pnt   AS INTEGER
END TYPE
