;-------------------------------
; FPrint - writes a string directly to the video buffer
; ~~~~~~
; 1.0 -- Original routine by Robert Church
; 1.1 -- Code optimized by Victor Yiu
;
; Basic declare:
; DECLARE SUB FPrint (S$, BYVAL Attr%, BYVAL Row%, BYVAL Col%)

CODE SEGMENT PARA PUBLIC 'CODE'
    PUBLIC FPrint
    ASSUME CS:code, DS:nothing, ES:nothing

FPrint PROC FAR

    JMP SHORT Start     ; skip over variable

EVEN
VidSegment DW 0

; ============ routine starts here
Start:
        ; BP+6  = Col
        ; BP+8  = Row
        ; BP+10 = Attr
        ; BP+12 = String pointer

    PUSH BP
    MOV BP, SP      ; set up stack frame

    PUSH SI         ; save needed registers
    PUSH DI

    LES BX, [BP+6]  ; get col. into BX, row into ES
    MOV AX, ES      ; ES --> AX; row into AX

    Dec   AX             ; (160 * (row - 1)) + (2 * (col - 1))

    MOV CL, 5       ; set CL up for bit-rotation
    SHL AX, CL      ; AX * 32
    MOV DX, AX      ; save AX*32
    SHL AX, 1
    SHL AX, 1       ; AX*4, for a total of AX*128
    ADD AX, DX      ; AX = AX*(128+32), which = AX*160!!!

    Dec   BX
    Shl   BX, 1
    Add   AX, BX
    Mov   DI, AX         ; the offset into video memory in which to
                         ; print is now in DI

    Mov  AX, CS:VidSegment ; figure out if the routine has been called yet
    OR AX, AX            ; okay?
    JNZ  SetupLoop       ; skip over init.

GetVidSegment:      ; ====== routine for finding out vid. segment
    Mov   ES, AX         ; 0 --> ES
    CMP BYTE PTR ES:[464h], 0B4h
    Jne   Color             ; if not equal, color
    MOV AH, 0B0h            ;Set the VidSegment to 0B000h, monochrome
    MOV CS:VidSegment, AX
    Jmp   SHORT SetupLoop

Color:
    Mov   AH, 0B8h             ;color
    Mov   CS:VidSegment, AX    ; either way, load BX with the segment

SetupLoop:
    Mov   ES, AX         ; load the video buffer's segment into ES

    MOV   AH, [BP+10]    ; put the attribute in AX
    MOV   SI, [BP+12]    ; load the QB string, length in CX, ptr in SI

    Mov   CX, [SI]       ; get the length
    Mov   SI, [SI+2]     ; get the starting point
    CLD                  ; go forward

    CMP CX, 80           ; >80?
    JAE WriteStart       ; just draw 80... jump to start of loop

    ; Current register contents:
    ;  AX = Attribute word
    ;  CX = string length
    ;  SI = pointer to string
    ;  DI = offset into video memory at which to print
    ;  DS = string segment (DGROUP)
    ;  ES = video memory segment

CalcStart:
    MOV DX, OFFSET ExitProc     ; find the ending place
    SHL CX, 1                   ; since MOVSB/STOSB is 2 bytes...
    SUB DX, CX                  ; find offset from end
    JMP DX                      ; jump; start drawing!

EVEN
WriteStart:

REPT 80
    LODSB                ; get the byte from string
    STOSW                ; write the byte and attribute to video
ENDM

ExitProc:
    POP DI      ; restore registers
    POP SI
    POP BP

    RET 8       ; return, removing the passed in variables

FPRINT ENDP
    CODE ENDS
    END
END   ;------------------------------- That's it...

