;========================================================= 
;
;                                K B . S Y S
;
;                       By: David B. Berger -- 12/31/91
;                       Assembled with Microsoft's MASM
;
;
;          This device driver will disable the use of the <Ctrl><C>
;            and <Ctrl><Break> key combinations from the keyboard.
;
;=========================================================
CSEG     SEGMENT BYTE PUBLIC 'CODE'          
         ASSUME CS:CSEG,DS:CSEG,ES:CSEG
         ORG 00H
KB:
;=========================================================
;                  D R I V E R   H E A D E R   B L O C K
; 
;        bytes 0000H-0011H of the driver MUST be set up as follows:
;=========================================================
         DW 0FFFFH                      ; +0: Offset of next device
         DW 0FFFFH                      ; +2: Segment of next device
;---------------------------------------------------------          
         DW 8000H    
                                        ; +4: Device Attribute Word
;---------------------------------------------------------
         DW OFFSET STRAT                ; +6: Offset of "Strategy" routine 
         DW OFFSET INTRPT               ; +8: Offset of "Interrupt" routine
         DB 'KB      '                  ; +0Ah..+11h = Name of Driver
;=========================================================
;                     S T R A T E G Y   R O U T I N E 
;
;       Called by DOS to save the pointer (in ES:BX) 
;       to the Device Driver Request Header
;---------------------------------------------------------
STRAT:          
         MOV    CS:[HDROFS],BX          ; Save offset of Request Header
         MOV    CS:[HDRSEG],ES          ; Save segment of Request Header
         RETF                           ; Return to DOS
;=========================================================
;       Local Data Storage
;--------------------------------------------------------- 
HEADER   LABEL  DWORD
HDROFS   DW     0000
HDRSEG   DW     0000
;========================================================= 
;                  I N T E R R U P T   P R O C E D U R E
;
;       Called by DOS to process a driver request, as defined in the
;       Request Header.  The Request Header is found at the address
;       previously passed to the Strategy Routine.
;
;       Request Header format:
;       +00 = NUMB      Number of bytes in Request Header (byte)
;       +01 = UNIT      Unit Number of this request (byte) (BLOCK)
;       +02 = CMMD      Request Command Code (byte)
;       +03 = STAT      Returned Status (word)
;       +05 = RESRVED   Reserved by DOS (eight bytes)
;       +0D = MEDIA     Media descriptor (byte)
;       +0E = ADDR      Data Transfer Address (word:word)
;       +12 = COUNT     Byte or sector count (word)
;       +14 = SECT      Starting sector value (BLOCK DEVICE)
;=========================================================
;                    K B   I N T E R R U P T   L O G I C
;---------------------------------------------------------
INTRPT:
         PUSH   AX                      ; Save the entry registers          
         PUSH   BX
         PUSH   CX
         PUSH   DX
         PUSH   ES
         PUSH   DS
         PUSH   DI
         PUSH   SI
;---------------------------------------------------------
;       Retrieve the address of the Request Header and get the CMMD
;---------------------------------------------------------
         PUSH   CS          
         POP    DS
         LES    BX,CS:[HEADER]          ; ES:BX = [HDRSEG:HDROFS]
         MOV    AL,ES:[BX+2]            ; CMMD = HDRSEG:[HDROFS+2]
;---------------------------------------------------------
         CMP    AL,00                   ; If CMMD = "INIT" then
         JNZ    BADRTN
         JMP    INIT                    ; INITIALIZE
;--------------------------------------------------------- 
;       For any other CMMD, return an error code in the STATUS
;       parameter and return to DOS.
;---------------------------------------------------------
BADRTN:          
         MOV    WORD PTR ES:[BX+3],8103H
         JMP    RETURN
GUDRTN:
         MOV    WORD PTR ES:[BX+3],0100H
RETURN:
         POP    SI
         POP    DI
         POP    DS
         POP    ES
         POP    DX
         POP    CX
         POP    BX
         POP    AX
         RETF                           ; Return (Far)
;=========================================================
KYBINT   LABEL  NEAR
         PUSH   AX          
         IN     AL,60H
         CMP    AL,1DH                  ; Ctrl Key Make Code
         JNZ    KYBT02
         MOV    BYTE PTR CS:KYBCON,1
         JMP    KYBJMP
KYBT02:  CMP    AL,9DH                  ; Ctrl Key Break Code
         JNZ    KYBT04
         MOV    BYTE PTR CS:KYBCON,0
         JMP    KYBJMP
KYBT04:  CMP    BYTE PTR CS:KYBCON,0
         JZ     KYBJMP
         CMP    AL,2EH                  ; C Key Make Code
         JZ     KYBSKP
         CMP    AL,46H                  ; Scroll Lock Key Make Code
         JNZ    KYBJMP
KYBSKP:  IN     AL,61H
         MOV    AH,AL
         OR     AL,80H
         OUT    61H,AL
         XCHG   AH,AL
         OUT    61H,AL
;
         MOV    AL,20H
         OUT    20H,AL
;
         POP    AX
         IRET
KYBJMP:  POP    AX
         PUSHF
         DB     09AH                    ; Far call
KYBOIP   DW     0000
KYBOCS   DW     0000
         IRET
;
KYBCON   DB     00
;========================================================= 
RUNSIZ   EQU    $-KB
         DB     (0100H-RUNSIZ) DUP (00)
;=========================================================
;              I N I T I A L I Z A T I O N   L O G I C
;=========================================================
;       This logic is invoked when DOS installs the driver. (CMMD=0).
;       The initialization steps include: ;
;       1. Retreive the address for the keyboard interrupt.
;       2. Install the new interrupt Ctrl-key check.
;       3. Flush the keyboard buffer.
;       4. Turn Off Ctrl Key Status Bits in BIOS memory.
;
;       When done, the logic tells DOS where the "expendable" part of
;       the driver (the initialization logic itself) begins.  DOS will
;       then release the memory AFTER that address for its own use.
;---------------------------------------------------------
INIT     LABEL  NEAR
         PUSH   AX                      ; Save the registers
         PUSH   BX          
         PUSH   CX
         PUSH   DX
         PUSH   ES
;
         SUB    AX,AX
         MOV    ES,AX
         MOV    BX,24H                  ; ES:BX = 0000:0024
         MOV    AX,ES:[BX]              ; Int 9 Vector
         MOV    KYBOIP,AX
         MOV    AX,ES:[BX+2]
         MOV    KYBOCS,AX
         LEA    AX,KYBINT               ; Replace with CS:KYBINT
         MOV    WORD PTR ES:[BX],AX     ; Int Handler
         MOV    WORD PTR ES:[BX+2],CS
;
         MOV    AH,06H                  ; Flush Keyboard Buffer
         MOV    DL,0FFH
         INT    21H
;
         MOV    AX,40H                  ; Turn Off Ctrl Bits in BIOS memory
         MOV    ES,AX
         MOV    BX,17H
         MOV    AL,ES:[BX]          
         AND    AL,0FBH
         MOV    ES:[BX],AL
         MOV    AL,ES:[BX+1]
         AND    AL,0FEH
         MOV    ES:[BX+1],AL
;
         POP    ES                      ; Restore the registers
         POP    DX
         POP    CX
         POP    BX
         LEA    AX,INIT
         MOV    WORD PTR ES:[BX+0EH],AX
         MOV    WORD PTR ES:[BX+10H],CS
         POP    AX
         JMP    GUDRTN                  ; Return
;=========================================================
CSEG     ENDS
         END    KB
