'****************************************************************************
'**   PULLDOWN.BAS  - Pulldown menu system for QB45 & MS PDS71             **
'**    (rev 12)       by Mark H Butler, released December 1992             **
'****************************************************************************
'**                                                                        **
'**     This code is a model for a pulldown menu system written entirely   **
'**     in MS QuickBASIC. Although commenting is scanty throughout this    **
'**     code it really shouldn't be too hard to analyse and incorporate    **
'**     into your own programs. Remember to dimension and define the       **
'**     menu arrays to the items you want to appear in your pulldown       **
'**     selections and you're cooking.                                     **
'**                                                                        **
'**     Note: as written this code needs to be compiled to be run because  **
'**     of the call to B$ASSN ("MemCopy") which is a QB internal routine   **
'**     found in the runtime lib (BCOM45.LIB) but is not available for     **
'**     calling within either the QB or the QBX environment.               **
'**                                                                        **
'**     Further Note: this code is being released to the public and        **
'**     I am placing no restrictions as such on it for it's use, however   **
'**     if you do use it in a project for sale I wouldn't mind some small  **
'**     amount of recognition for having made this code available to you.  **
'**     That is, if my name shows up somewhere in the credits for your     **
'**     program I think that would be considerate of you.                  **
'**                                                                        **
'**      Happy coding,                                                     **
'**                                                                        **
'**     -*[-M-H-B-]*-                                                      **
'**                                                                        **
'****************************************************************************
DEFINT A-Z
CONST UpArrow = 72, DownArrow = 80, Escape = 27, Enter = 13, F1 = 59
CONST Home = 71, EndKey = 79, LeftArrow = 75, RightArrow = 77
DECLARE FUNCTION Center (text$)
DECLARE SUB BarMenu (row, col, spacing, BarPrompt$(), return$, startpos)
DECLARE SUB Burp ()
DECLARE SUB CheckMoves ()
DECLARE SUB ClearKbd ()
DECLARE SUB DeskTop ()
DECLARE SUB DrawBox (Uprow, Ltcol, Lorow, Rtcol)
DECLARE SUB Help ()
DECLARE SUB MenuBox (row, col, Item$(), ItemNum, MaxItems, movekey)
DECLARE SUB MemCopy ALIAS "B$ASSN" (BYVAL FSeg, BYVAL FOfs, BYVAL NBytes1, BYVAL TSeg, BYVAL TOfs, BYVAL NBytes2)
DECLARE SUB Pause (ticks)
DECLARE SUB PokeBar ()
DECLARE SUB RestScrn (Scr())
DECLARE SUB SaveScrn (Scr())
DECLARE SUB Shadow (Uprow, Ltcol, Lorow, Rtcol)
DIM BarPrompt$(1 TO 6), Menu1$(1 TO 9), Menu2$(1 TO 7)
DIM Menu3$(1 TO 12), Menu4$(1 TO 3), Menu5$(1 TO 6), Menu6$(1 TO 2)
DIM SHARED Scr(4000)

    DeskTop
    GOSUB FillMenus

    '*** initialize flags
    movekey = 0
    startpos = 1
    trigger = 0

    DO
        return$ = ""
        CALL BarMenu(1, 1, 4, BarPrompt$(), return$, startpos)
        HotCharPos = INSTR(HotChar$, return$)
        IF return$ = "aborted" THEN HotCharPos = 0
        SaveScrn Scr()
        SELECT CASE HotCharPos
            CASE 0
                RestScrn Scr()
                COLOR 7, 0
                CLS
                PRINT " user aborted..."
                END
            CASE 1
                CALL MenuBox(2, 2, Menu1$(), ItemNum, 9, movekey)
            CASE 2
                CALL MenuBox(2, 10, Menu2$(), ItemNum, 7, movekey)
            CASE 3
                CALL MenuBox(2, 18, Menu3$(), ItemNum, 12, movekey)
            CASE 4
                CALL MenuBox(2, 26, Menu4$(), ItemNum, 3, movekey)
            CASE 5
                CALL MenuBox(2, 37, Menu5$(), ItemNum, 6, movekey)
            CASE 6
                CALL MenuBox(2, 47, Menu6$(), ItemNum, 2, movekey)
        END SELECT
        CheckMoves
        IF ItemNum > 0 THEN
            LOCATE 25, 25
            COLOR 14, 4
            PRINT " You chose menu #"; HotCharPos; ", Item #"; ItemNum;
            Pause 5 * 18
            trigger = 0
            startpos = 1
            ClearKbd
        END IF
        RestScrn Scr()
    LOOP

'======================================================================
'               M E N U   D A T A
'======================================================================

FillMenus:
    
    BarPrompt$(1) = "Rend"
    BarPrompt$(2) = "Tear"
    BarPrompt$(3) = "Kill"
    BarPrompt$(4) = "Destroy"
    BarPrompt$(5) = "Mayhem"
    BarPrompt$(6) = "Pillage"

    HotChar$ = "RTKDMP" '<---*** first letter of each barmenu item

    FOR i = 1 TO 9
        READ Menu1$(i)
    NEXT
    DATA First Menu1 Option, Second Menu1 Option,Third Menu1 Option
    DATA Fourth Menu1 Option,Fifth Menu1 Option,Sixth Menu1 Option
    DATA Seventh Menu1 Option,Eighth Menu1 Option,Ninth Menu1 Option

    FOR i = 1 TO 7
        READ Menu2$(i)
    NEXT
    DATA First Menu2 Option, Second Menu2 Option,Third Menu2 Option
    DATA Fourth Menu2 Option,Fifth Menu2 Option,Sixth Menu2 Option
    DATA Seventh Menu2 Option

    FOR i = 1 TO 12
        READ Menu3$(i)
    NEXT
    DATA First Menu3 Option, Second Menu3 Option,Third Menu3 Option
    DATA Fourth Menu3 Option,Fifth Menu3 Option,Sixth Menu3 Option
    DATA Seventh Menu3 Option,Eighth Menu3 Option,Ninth Menu3 Option
    DATA Tenth Menu3 Option,Eleventh Menu3 Option,Twelfth Menu3 Option

    FOR i = 1 TO 3
        READ Menu4$(i)
    NEXT
    DATA First Menu4 Option, Second Menu4 Option,Third Menu4 Option

    FOR i = 1 TO 6
        READ Menu5$(i)
    NEXT
    DATA First Menu5 Option, Second Menu5 Option,Third Menu5 Option
    DATA Fourth Menu5 Option,Fifth Menu5 Option,Sixth Menu5 Option

    FOR i = 1 TO 2
        READ Menu6$(i)
    NEXT
    DATA First Menu6 Option, Second Menu6 Option

    RETURN

'*------[End Of Main]------*

SUB BarMenu (row, col, spacing, prompt$(), return$, startpos)
SHARED trigger

    first = LBOUND(prompt$)
    last = UBOUND(prompt$)
    REDIM Position(first TO last)

'*** Create the menu bar and remember the 'hotchars' (first letter
'*** of each drop menu heading.

    IF NOT trigger THEN
        COLOR 0, 7
        LOCATE row, col, 0
        PRINT SPACE$(80);
    END IF
    LOCATE row, col
    PRINT "   ";


    FOR i = first TO last
        Position(i) = POS(1)
        IF NOT trigger THEN
            COLOR 15, 7
        ELSE
            COLOR 0, 7
        END IF
        PRINT LEFT$(prompt$(i), 1);
        COLOR 0, 7
        PRINT RIGHT$(prompt$(i), LEN(prompt$(i)) - 1); SPC(spacing);
        HotChar$ = HotChar$ + CHR$(ASC(prompt$(i)))
    NEXT i

'*** Move around on the bar and get user intput.
'*** Valid keys: Right-arrow, Left-arrow, Home & End keys to move.
'*** Enter, First letter of menu option and Down-arrow to choose.
'*** Escape key to abort.

    current = startpos
    moveto = current
    return$ = ""
    DO
        LOCATE row, Position(current) - 1
        COLOR 7, 0
        PRINT " "; prompt$(current); " "
        IF return$ <> "" THEN
            EXIT SUB
        END IF
        IF trigger THEN
            ch$ = CHR$(0) + "P"
        ELSE
            ch$ = ""
        END IF
        DO WHILE ch$ = ""
            ch$ = UCASE$(INKEY$)
        LOOP
        IF ASC(ch$) = 0 THEN
            SpecialKey = ASC(RIGHT$(ch$, 1))
            SELECT CASE SpecialKey
                CASE 59
                    Help
                CASE DownArrow
                    PokeBar
                    trigger = -1
                    return$ = CHR$(ASC(prompt$(current)))
                CASE Home
                    moveto = first
                CASE EndKey
                    moveto = last
                CASE LeftArrow
                    moveto = moveto - 1
                CASE RightArrow
                    moveto = moveto + 1
                CASE ELSE
                Burp
            END SELECT
            IF moveto < first THEN moveto = last
            IF moveto > last THEN moveto = first
        ELSEIF ch$ = CHR$(13) THEN
            trigger = -1
            return$ = CHR$(ASC(prompt$(current)))
        ELSEIF ch$ = CHR$(27) THEN
            return$ = "aborted"
        ELSEIF INSTR(HotChar$, ch$) > 0 THEN
            PokeBar
            return$ = ch$
            moveto = INSTR(HotChar$, ch$)
            trigger = -1
        ELSE
            Burp
        END IF
        IF moveto <> current AND NOT trigger THEN
            LOCATE row, Position(current) - 1
            IF NOT trigger THEN
                COLOR 15, 7
            ELSE
                COLOR 0, 7
            END IF
            PRINT " "; LEFT$(prompt$(current), 1);
            COLOR 0, 7
            PRINT RIGHT$(prompt$(current), LEN(prompt$(current)) - 1); SPC(spacing);
        END IF
        current = moveto
        IF trigger THEN PokeBar
        ch$ = ""
    LOOP
    trigger = -1
END SUB

SUB Burp
    SOUND 70, .5
    SOUND 37, .5
END SUB

FUNCTION Center (text$)
    Center = ((80 - LEN(text$)) \ 2)
END FUNCTION

SUB CheckMoves
SHARED trigger, startpos, HotCharPos, movekey, return$, ItemNum
    IF movekey THEN
        trigger = -1
        startpos = HotCharPos + movekey
        IF startpos > 6 THEN startpos = 1
        IF startpos < 1 THEN startpos = 6
        movekey = 0
        return$ = ""
        ItemNum = 0
    END IF
END SUB

SUB ClearKbd
    DO UNTIL INKEY$ = "": LOOP
END SUB

SUB DeskTop

    COLOR 7, 1
    CLS
    DEF SEG = &H40
    equip = PEEK(&H10)
    IF (equip AND 48) = 48 THEN
        DEF SEG = &HB000
    ELSE
        DEF SEG = &HB800
    END IF

    FOR X = 0 TO 4000 STEP 2
        POKE X, 32
    NEXT

    DEF SEG

    DrawBox 2, 1, 25, 80

    FileName$ = " Untitled "
    LOCATE 2, Center(FileName$)
    COLOR 1, 7
    PRINT FileName$;
    LOCATE 2, 76
    COLOR 7, 1
    PRINT CHR$(180);
    COLOR 1, 7
    PRINT CHR$(24);
    COLOR 7, 1
    PRINT CHR$(195);

    COLOR 0, 3
    LOCATE 25, 1
    PRINT SPACE$(80);
    LOCATE 25, 68
    PRINT "  F1 Help  ";
    
END SUB

SUB DrawBox (Uprow, Ltcol, Lorow, Rtcol)
    Wide = (Rtcol - Ltcol) - 1
    LOCATE Uprow, Ltcol
    PRINT CHR$(218); STRING$(Wide, CHR$(196)); CHR$(191);
    FOR i = Uprow + 1 TO Lorow - 1
        LOCATE i, Ltcol
        PRINT CHR$(179); '*** SPACE$(Wide); <- for filled box
        LOCATE i, Rtcol
        PRINT CHR$(179);
    NEXT
    LOCATE Lorow, Ltcol
    PRINT CHR$(192); STRING$(Wide, CHR$(196)); CHR$(217);
END SUB

SUB Help
    REDIM HelpScrn(4000)
    SaveScrn HelpScrn()
    COLOR 0, 7
    DrawBox 10, 21, 13, 65
    LOCATE 11, 22: PRINT " This program needs a whole lot less help  "
    LOCATE 12, 22: PRINT " than it did before, that much is certain! "
    Shadow 10, 21, 13, 65
    COLOR 7, 0
    Pause 6 * 18
    RestScrn HelpScrn()
    ClearKbd
END SUB

SUB MenuBox (row, col, Item$(), ItemNum, MaxItems, movekey)
SHARED trigger, startpos

    row = row + 1
    col = col + 1
    ItemNum = 1
    ItemLength = 1
    FOR i = 1 TO MaxItems
    IF LEFT$(Item$(i), 1) <> " " THEN
        Item$(i) = " " + Item$(i) + " "
    END IF
    NEXT i
    FOR i = 1 TO MaxItems
        IF LEN(Item$(i)) > ItemLength THEN
            ItemLength = LEN(Item$(i))
        ELSEIF LEN(Item$(i)) < 14 THEN
           ItemLength = 14
        END IF
    NEXT i

'*******************************************
'***** build frame around choices **********
'*******************************************
       
    Ltcol = col - 1
    Rtcol = col + ItemLength
    COLOR 0, 7

    LOCATE row - 1, Ltcol
    PRINT CHR$(218);
    FOR i = 1 TO ItemLength     '**** top of frame ****
        PRINT CHR$(196);
    NEXT i
    PRINT CHR$(191)
  
    LOCATE row, 1
    FOR i = 1 TO MaxItems    '***  print menu items and frame sides **
        LOCATE , Ltcol
        PRINT CHR$(179); TAB(Rtcol); CHR$(179)
    NEXT i

    LOCATE row + MaxItems, Ltcol
    PRINT CHR$(192);
    FOR i = 1 TO ItemLength      '*** bottom1 of frame ****
        PRINT CHR$(196);
    NEXT i
    PRINT CHR$(217);
    Shadow row - 1, Ltcol, row + MaxItems, Rtcol

'*******************************************
'******** print selections and *************
'******** pointer **************************
'*******************************************

    LOCATE row
    FOR i = 1 TO MaxItems
        LOCATE row + i - 1, col
        PRINT Item$(i)
    NEXT i

    DO
        COLOR 7, 0       '************************* Hilite color
      
        LOCATE row + ItemNum - 1, col
        Item$(ItemNum) = Item$(ItemNum) + SPACE$(ItemLength - LEN(Item$(ItemNum)))
        PRINT Item$(ItemNum)

        COLOR 0, 7       '******* back to normal item color

'*******************************************
'***** Start picking up the keystrokes *****
'*******************************************

        DO WHILE ch$ = ""
            ch$ = UCASE$(INKEY$)
        LOOP
        kee = ASC(ch$)
        IF ASC(ch$) = 0 THEN
            kee = ASC(RIGHT$(ch$, 1))
        END IF

        LOCATE row + ItemNum - 1, col
        PRINT Item$(ItemNum)
        
        SELECT CASE kee
            CASE F1
                Help
            CASE LeftArrow
                movekey = -1
                EXIT SUB
            CASE RightArrow
                movekey = 1
                EXIT SUB
            CASE Home
                ItemNum = 1
            CASE EndKey
                ItemNum = MaxItems
            CASE UpArrow
                IF ItemNum > 1 THEN
                    ItemNum = ItemNum - 1
                ELSE
                    ItemNum = MaxItems
                END IF
            CASE DownArrow
                IF ItemNum < MaxItems THEN
                    ItemNum = ItemNum + 1
                ELSE
                    ItemNum = 1
                END IF
            CASE Enter
                fore = 0
                back = 7
                FOR i = 1 TO 6
                    COLOR fore, back
                    IF fore = 7 THEN
                        fore = 0: back = 7
                    ELSE
                        fore = 7: back = 0
                    END IF
                    LOCATE row + ItemNum - 1, col
                    Item$(ItemNum) = Item$(ItemNum) + SPACE$(ItemLength - LEN(Item$(ItemNum)))
                    PRINT Item$(ItemNum)
                    Pause 1
                NEXT
                EXIT DO
            CASE Escape
                trigger = 0
                ItemNum = 0
                startpos = 1
                EXIT DO
            CASE ELSE
                Burp
        END SELECT
        ch$ = ""
    LOOP
    ClearKbd
END SUB

SUB Pause (ticks)
    DEF SEG = 0
    DO UNTIL TestTick > ticks
        IF LEN(INKEY$) THEN EXIT SUB
        LastTick = GetTick
        GetTick = PEEK(&H46C)
        IF LastTick <> GetTick THEN
            TestTick = TestTick + 1
        END IF
    LOOP
    DEF SEG
END SUB

SUB PokeBar

    DEF SEG = &H40
    equip = PEEK(&H10)
    IF (equip AND 48) = 48 THEN
        DEF SEG = &HB000
    ELSE
        DEF SEG = &HB800
    END IF

    row = 1

    FOR column = 1 TO 80
        offset = (row - 1) * 160 + (column - 1) * 2 + 1
        POKE offset, 112
    NEXT
    DEF SEG
END SUB

SUB RestScrn (Scr())
    DEF SEG = &H40
    equip = PEEK(&H10)
    IF (equip AND 48) = 48 THEN
        destseg = &HB000
    ELSE
        destseg = &HB800
    END IF
    destoff = 0
    fromseg = VARSEG(Scr(0))
    fromoff = VARPTR(Scr(0))
    nbytes = 4000
    MemCopy fromseg, fromoff, nbytes, destseg, destoff, nbytes
END SUB

SUB SaveScrn (Scr())
    DEF SEG = &H40
    equip = PEEK(&H10)
    IF (equip AND 48) = 48 THEN
        fromseg = &HB000
    ELSE
        fromseg = &HB800
    END IF
    fromoff = 0
    destseg = VARSEG(Scr(0))
    destoff = VARPTR(Scr(0))
    nbytes = 4000
    MemCopy fromseg, fromoff, nbytes, destseg, destoff, nbytes
END SUB

SUB Shadow (Uprow, Ltcol, Lorow, Rtcol)
    DEF SEG = &H40
    equip = PEEK(&H10)
    IF (equip AND 48) = 48 THEN
        EXIT SUB
    ELSE
        DEF SEG = &HB800
    END IF
    FOR vert = Uprow + 1 TO Lorow + 1
        FOR height = Rtcol + 1 TO Rtcol + 2
            offset = (vert - 1) * 160 + (height - 1) * 2 + 1
            POKE offset, 8
        NEXT
    NEXT
    vert = Lorow + 1
    FOR height = Ltcol + 2 TO Rtcol + 2
        offset = (vert - 1) * 160 + (height - 1) * 2 + 1
        POKE offset, 8
    NEXT
    DEF SEG
END SUB

