/*
 * uuencode [input [output] ]
 *
 * if output not specified, output to stdout
 * if input not specified, input from stdin
 *
 * Encode a file so it can be mailed to a remote system.
 *
 * Modified for use with Microsoft C and VAX/VMS.
 * Define CI86 symbol to use with CI86.
 */
#ifndef VMS
#include <stdio.h>
#ifndef	CI86
#include <sys/types.h>
#include <sys/stat.h>
#endif
#else	/* ifndef VMS */
#include stdio
#include types
#include stat
#endif	/* ifndef VMS */

/* ENC is the basic 1 character encoding function to make a char
printing */
#define ENC(c) (((c) & 077) + ' ')

main(argc, argv)
char **argv;
{
	FILE *in, *out;
#ifndef	CI86
	struct stat sbuf;
#endif
	int mode;

	/* if 3 arguments, then output file specified */
	if (argc > 2) {
		if ((out = fopen(argv[2], "w")) == NULL) {
	    		perror(argv[2]);
			exit(3);
		}
		argc--;
	}
	else
		out = stdout;
	if (argc > 1) {
#ifdef	MSDOS
		/* Use binary mode */
		if ((in = fopen(argv[1], "rb")) == NULL) {
#else
		if ((in = fopen(argv[1], "r")) == NULL) {
#endif
			perror(argv[1]);
			exit(1);
		}
		argc--;
	} else
		in = stdin;

	if (argc != 1) {
		fprintf(stderr,"Usage: uuencode [infile [outfile] ]\n");
		exit(2);
	}

#ifndef	CI86
	/* figure out the input file mode */
	fstat(fileno(in), &sbuf);
	mode = sbuf.st_mode & 0777;
#else
	mode = 0;  /* default to normal mode */
#endif
	fprintf(out,"begin %o %s\n", mode, argv[1]);

	encode(in, out);

	fprintf(out,"end\n");
#ifndef VMS
	exit(0);
#else
	exit(1);	/* VMS successful */
#endif
}

/*
 * copy from in to out, encoding as you go along.
 */
encode(in, out)
FILE *in;
FILE *out;
{
	char buf[80];
	int i, n;

	for (;;) {
		/* 1 (up to) 45 character line */
		n = fr(in, buf, 45);
		putc(ENC(n), out);

		for (i=0; i<n; i += 3)
			outdec(&buf[i], out);

		putc('\n', out);
		if (n <= 0)
			break;
	}
}

/*
 * output one group of 3 bytes, pointed at by p, on file f.
 */
outdec(p, f)
char *p;
FILE *f;
{
	int c1, c2, c3, c4;

	c1 = *p >> 2;
	c2 = (*p << 4) & 060 | (p[1] >> 4) & 017;
	c3 = (p[1] << 2) & 074 | (p[2] >> 6) & 03;
	c4 = p[2] & 077;
	putc(ENC(c1), f);
	putc(ENC(c2), f);
	putc(ENC(c3), f);
	putc(ENC(c4), f);
}

/* fr: like read but stdio */
int
fr(fd, buf, cnt)
FILE *fd;
char *buf;
int cnt;
{
	int c, i;

	for (i=0; i<cnt; i++) {
		c = getc(fd);
		if (c == EOF)
			return(i);
		buf[i] = c;
	}
	return (cnt);
}
